#include <zephyr/types.h>
#include <stddef.h>
#include <errno.h>
#include <zephyr/kernel.h>
#include <zephyr/sys/byteorder.h>
#include <zephyr/logging/log.h>
#include <stdbool.h>
#include <zephyr/dfu/mcuboot.h>
#include <helpers/nrfx_reset_reason.h>

#include <zephyr/storage/flash_map.h>
#include <zephyr/drivers/flash.h>
#include <flash_map_pm.h>

#include "ble_scanner_new.h"
#include "sensor_dfu_handler.h"
#include "image_hash_utils.h"

#define LOG_MODULE_NAME nrf_central
LOG_MODULE_REGISTER(LOG_MODULE_NAME);

#define LED_BLINKING_INTERVAL K_SECONDS(1)


static uint8_t get_reset_reason(void)
{
	uint8_t res = 0xff;
	uint32_t temp = nrfx_reset_reason_get();

	nrfx_reset_reason_clear(temp);

	res = (uint8_t) temp;

	if (temp & NRFX_RESET_REASON_RESETPIN_MASK) {
		LOG_INF("pin-reset");
	}

	if (temp & NRFX_RESET_REASON_DOG_MASK) {
		LOG_INF("watchdog/application watchdog timer 0 reset");
	}

	if (temp & NRFX_RESET_REASON_SREQ_MASK) {
		LOG_INF("soft reset/application soft reset");
	}

	if (temp & NRFX_RESET_REASON_LOCKUP_MASK) {
		LOG_INF("CPU lockup/application CPU lockup reset");
	}

	if (temp & NRFX_RESET_REASON_OFF_MASK) {
		LOG_INF("wakeup from System OFF triggered by DETECT signal");
	}

	if (temp & NRFX_RESET_REASON_LPCOMP_MASK) {
		LOG_INF("wakeup from System OFF triggered by ANADETECT signal");
	}

	if (temp & NRFX_RESET_REASON_DIF_MASK) {
		LOG_INF("wakeup from System OFF triggered by entering the debug interface mode");
	}

	if (temp & NRFX_RESET_REASON_NFC_MASK) {
		LOG_INF("wakeup from System OFF mode due to NRF field");
	}

	if (temp & NRFX_RESET_REASON_VBUS_MASK) {
		LOG_INF("wakeup from System OFF mode due to VBUS valid");
	}

	if (temp == 0) {
		/* If none of the reset sources is flagged, the chip was reset from the on-chip reset generator.
		 * This indicates a power-on-reset or a brown out reset.
		 * (from modules/hal/nordic/nrfx/helpers/nrfx_reset_reason.h)
		 */
		LOG_INF("probably power-on or a brown-out reset");
	}

	return res;
}

int main(void)
{
	uint8_t reset_reason;
	
	//nrf_profiler_init();

    LOG_INF("Main application started (%02d.%02d.%02d)!", VERSION_MAJOR, VERSION_MINOR, VERSION_PATCH);
	reset_reason = get_reset_reason();

    boot_write_img_confirmed();


	if(ble_scanner_init(NULL, false) == 0){

	} else {
		LOG_ERR("could not initialize bluetooth scanner");
		goto error;
	}

	// This must be set to the mac of the periperial device.
	// I had f9:67:9f:b2:04:bc
	uint8_t addr[6] = {0xbc, 0x04, 0xb2, 0x9f, 0x67, 0xf9};
	LOG_INF("Adding sensor %02x:%02x:%02x:%02x:%02x:%02x for listening", addr[5], addr[4], addr[3], addr[2], addr[1], addr[0]);
	add_connectable_addr(addr);
    
	return 0;
error:
	LOG_ERR("Could not start NRF");
	return -1;
}
